"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const windowState = require("electron-window-state");
const menubar = require("menubar");
const account_switcher_1 = require("./account_switcher");
const log_1 = require("./log");
const common_1 = require("./common");
class Window {
    constructor(browser, state, account, menubar) {
        this.browser = browser;
        this.state = state;
        this.account = account;
        this.menubar = menubar;
        if (!common_1.IS_DARWIN) {
            browser.setMenu(electron_1.Menu.getApplicationMenu());
        }
        browser.webContents.on('will-navigate', (e, url) => {
            if (!url.startsWith(`https://${this.account.host}`)) {
                e.preventDefault();
                electron_1.shell.openExternal(url);
            }
            log_1.default.debug('Opened URL with external browser (will-navigate)', url);
        });
        browser.webContents.on('new-window', (e, url) => {
            e.preventDefault();
            electron_1.shell.openExternal(url);
            log_1.default.debug('Opened URL with external browser (new-window)', url);
        });
        browser.webContents.session.setPermissionRequestHandler((contents, permission, callback) => {
            const url = contents.getURL();
            const grantedByDefault = url.startsWith(`https://${this.account.host}`) &&
                permission !== 'geolocation' &&
                permission !== 'media';
            if (grantedByDefault) {
                log_1.default.debug('Permission was granted', permission);
                callback(true);
                return;
            }
            log_1.default.debug('Create dialog for user permission', permission);
            electron_1.dialog.showMessageBox({
                type: 'question',
                buttons: ['Accept', 'Reject'],
                message: `Permission '${permission}' is requested by ${url}`,
                detail: "Please choose one of 'Accept' or 'Reject'",
            }, (buttonIndex) => {
                const granted = buttonIndex === 0;
                callback(granted);
            });
        });
    }
    static create(account, config, mb = null) {
        if (config.normal_window) {
            return startNormalWindow(account, config);
        }
        else {
            return startMenuBar(account, config, mb);
        }
    }
    open(url) {
        log_1.default.debug('Open URL:', url);
        this.browser.loadURL(url);
    }
    close() {
        log_1.default.debug('Closing window:', this.account);
        this.state.unmanage();
        this.browser.webContents.removeAllListeners();
        this.browser.removeAllListeners();
        if (this.menubar) {
            delete this.menubar.window;
        }
        this.browser.close();
    }
}
exports.default = Window;
function startNormalWindow(account, config) {
    log_1.default.debug('Setup a normal window');
    return new Promise(resolve => {
        const state = windowState({
            defaultWidth: 600,
            defaultHeight: 800,
        });
        const win = new electron_1.BrowserWindow({
            width: state.width,
            height: state.height,
            x: state.x,
            y: state.y,
            icon: common_1.APP_ICON,
            show: false,
            useContentSize: true,
            autoHideMenuBar: true,
            webPreferences: {
                nodeIntegration: false,
                sandbox: true,
                preload: common_1.PRELOAD_JS,
                partition: account_switcher_1.partitionForAccount(account),
            },
        });
        win.once('ready-to-show', () => {
            win.show();
        });
        win.once('closed', () => {
            electron_1.app.quit();
        });
        if (state.isFullScreen) {
            win.setFullScreen(true);
        }
        else if (state.isMaximized) {
            win.maximize();
        }
        state.manage(win);
        win.webContents.on('dom-ready', () => {
            log_1.default.debug('Send config to renderer procress');
            win.webContents.send('mstdn:config', config, account);
        });
        win.webContents.once('dom-ready', () => {
            log_1.default.debug('Normal window application was launched');
            if (common_1.IS_DEBUG) {
                win.webContents.openDevTools({ mode: 'detach' });
            }
        });
        resolve(new Window(win, state, account, null));
    });
}
function startMenuBar(account, config, bar) {
    log_1.default.debug('Setup a menubar window');
    return new Promise(resolve => {
        const state = windowState({
            defaultWidth: 350,
            defaultHeight: 420,
        });
        const icon = common_1.trayIcon(config.icon_color);
        const mb = bar || menubar({
            icon,
            width: state.width,
            height: state.height,
            alwaysOnTop: common_1.IS_DEBUG || !!config.always_on_top,
            tooltip: 'Mstdn',
            useContentSize: true,
            autoHideMenuBar: true,
            show: false,
            showDockIcon: true,
            webPreferences: {
                nodeIntegration: false,
                sandbox: true,
                preload: common_1.PRELOAD_JS,
                partition: account_switcher_1.partitionForAccount(account),
            },
        });
        mb.once('after-create-window', () => {
            log_1.default.debug('Menubar application was launched');
            if (common_1.IS_DEBUG) {
                mb.window.webContents.openDevTools({ mode: 'detach' });
            }
            mb.window.webContents.on('dom-ready', () => {
                log_1.default.debug('Send config to renderer procress');
                mb.window.webContents.send('mstdn:config', config, account);
            });
            state.manage(mb.window);
            resolve(new Window(mb.window, state, account, mb));
        });
        mb.once('after-close', () => {
            electron_1.app.quit();
        });
        if (bar) {
            log_1.default.debug('Recreate menubar window with different partition:', account);
            const pref = mb.getOption('webPreferences');
            pref.partition = account_switcher_1.partitionForAccount(account);
            mb.setOption('webPreferences', pref);
            mb.showWindow();
        }
        else {
            log_1.default.debug('New menubar instance was created:', account);
            mb.once('ready', () => mb.showWindow());
        }
    });
}
//# sourceMappingURL=window.js.map